from unittest import TestCase

from storage import StorageRepository
from team_domain import TeamDomain

class TestTeamDomain(TestCase):
    def setUp(self):
        with open('data/P1.csv', 'r') as file:
            self.files = [file.readlines()]
        self.storage = StorageRepository(self.files)
        self.storage.build()
        self.team_domain = TeamDomain()

    def test_get_all_teams(self):
        all_teams = self.team_domain.get_all_teams()
        self.assertIsInstance(all_teams, list)
        self.assertTrue(len(all_teams) == 18)

    def test_get_all_matches_by_team_name(self):
        matches = self.team_domain.get_all_matches_by_team_name('BENFICA')
        self.assertIsInstance(matches, list)
        self.assertTrue(len(matches) == 34)

    def test_get_all_home_matches_by_team_name(self):
        matches = self.team_domain.get_all_home_matches_by_team_name('BENFICA')
        self.assertIsInstance(matches, list)
        self.assertTrue(len(matches) == 17)

    def test_get_all_away_matches_by_team_name(self):
        matches = self.team_domain.get_all_away_matches_by_team_name('BENFICA')
        self.assertIsInstance(matches, list)
        self.assertTrue(len(matches) == 17)

    def test_get_all_home_matches_winner_by_team_name(self):
        matches = self.team_domain.get_all_home_matches_winner_by_team_name('BENFICA')
        self.assertIsInstance(matches, list)
        self.assertTrue(len(matches) > 0)

    def test_get_all_home_matches_lose_by_team_name(self):
        matches = self.team_domain.get_all_home_matches_lose_by_team_name('BENFICA')
        self.assertIsInstance(matches, list)
        self.assertTrue(len(matches) > 0)

    def test_get_all_home_matches_draw_by_team_name(self):
        matches = self.team_domain.get_all_home_matches_draw_by_team_name('BENFICA')
        self.assertIsInstance(matches, list)
        self.assertTrue(len(matches) > 0)

    def test_get_all_away_matches_winner_by_team_name(self):
        matches = self.team_domain.get_all_away_matches_winner_by_team_name('BENFICA')
        self.assertIsInstance(matches, list)
        self.assertTrue(len(matches) > 0)

    def test_get_all_away_matches_lose_by_team_name(self):
        matches = self.team_domain.get_all_away_matches_lose_by_team_name('BENFICA')
        self.assertIsInstance(matches, list)
        self.assertTrue(len(matches) > 0)

    def test_get_all_away_matches_draw_by_team_name(self):
        matches = self.team_domain.get_all_away_matches_draw_by_team_name('BENFICA')
        self.assertIsInstance(matches, list)
        self.assertTrue(len(matches) > 0)

    def test_get_all_matches_winner_by_team_name(self):
        matches = self.team_domain.get_all_matches_winner_by_team_name('BENFICA')
        self.assertIsInstance(matches, list)
        self.assertTrue(len(matches) == 28)

    def test_get_all_matches_lose_by_team_name(self):
        matches = self.team_domain.get_all_matches_lose_by_team_name('BENFICA')
        self.assertIsInstance(matches, list)
        self.assertTrue(len(matches) == 3)

    def test_get_all_matches_draw_by_team_name(self):
        matches = self.team_domain.get_all_matches_draw_by_team_name('BENFICA')
        self.assertIsInstance(matches, list)
        self.assertTrue(len(matches) == 3)


from unittest import TestCase

from storage import StorageRepository

class TestStorage(TestCase):
    def setUp(self):
        with open('data/P1.csv', 'r') as file:
            self.files = [file.readlines()]
        self.storage = StorageRepository(self.files)

    def test_build(self):
        self.storage.build()
        self.assertTrue(len(self.storage.storage) != 0)






from team_domain import TeamDomain

class Statistics(TeamDomain):
    def count_total_matches_by_team(self, team):
        return len(super().get_all_matches_by_team_name(team))

    def count_total_matches_winner_by_team(self, team):
        return len(super().get_all_matches_winner_by_team_name(team))
    
    def count_total_matches_lose_by_team(self, team):
        return len(super().get_all_matches_lose_by_team_name(team))

    def count_total_matches_draw_by_team(self, team):
        return len(super().get_all_matches_draw_by_team_name(team))

    def percentage_of_matches_winner_by_team(self, team):
        return 

    def percentage_of_matches_lose_by_team(self, team):
        return 

    def percentage_of_matches_draw_by_team(self, team):
        return 

    def __init__(self, team):
        self.team_name = self.storage[team.lower()]['team_name']
        self.total_matches_winner = self.count_total_matches_winner_by_team(team)
        self.total_matches_lose = self.count_total_matches_lose_by_team(team)
        self.total_matches_draw = self.count_total_matches_draw_by_team(team)
        self.total_matches = self.count_total_matches_by_team(team)
        self.percentage_of_matches_winner = 0
        self.percentage_of_matches_lose = 0
        self.percentage_of_matches_draw = 0
        



from collections import defaultdict

class StorageRepository():
    storage = defaultdict()

    def parse_file(self, lines):
        for line in lines:
            if lines.index(line) == 0:
                pass
            else:
                line_parsed = line.split(',')

                if len(line_parsed) > 3:
                    if line_parsed[2].lower() not in self.storage.keys():
                        self.storage[line_parsed[2].lower()] = defaultdict(list)
                    if line_parsed[3].lower() not in self.storage.keys():
                        self.storage[line_parsed[3].lower()] = defaultdict(list)

                    if line_parsed in self.storage[line_parsed[2].lower()]['matches']:
                        continue
                    if line_parsed in self.storage[line_parsed[3].lower()]['matches']:
                        continue

                    def get_team_name(param):
                        return [x for x in line_parsed if x.lower() == line_parsed[param].lower()][0]

                    self.storage[line_parsed[2].lower()]['matches'].append(line_parsed)
                    self.storage[line_parsed[2].lower()]['team_name'] = get_team_name(2)
                    self.storage[line_parsed[3].lower()]['matches'].append(line_parsed)
                    self.storage[line_parsed[3].lower()]['team_name'] = get_team_name(3)
                else:
                    pass
     
    def build(self):
        for file in self.files:
            self.parse_file(file)
              
    def __init__(self, csv_files):
        try:
            self.files = csv_files
            self.build()
        except Exception as ex:
            print("Can't build map.", ex)



import glob
import os
import shutil
from storage import StorageRepository
from statistics import Statistics
from time import time
from bs4 import BeautifulSoup
import requests
from requests.packages.urllib3.exceptions import InsecureRequestWarning
import pickle
import click
import shutil

def get_files(url, ext='csv'):
        try:
            page = requests.get(url).text
            soup = BeautifulSoup(page, 'html.parser')
            return ["/".join(url.split('/')[:3]) + '/' + node.get('href') for node in soup.find_all('a') if node.get('href').endswith(ext)]
        except:
            return "Invalid Url."

def decode_files(files):
    files_output = []

    for rfile in files:
        with open(rfile, 'r') as file:
            files_output.append(file.readlines())
    return StorageRepository(files_output)

def get_map():
    try:
        with open('.map', 'rb') as file:
            files = pickle.load(file)
        return decode_files(files)
    except TypeError:
        memory_files = glob.glob("data/*.csv")
        if len(memory_files) != 0:
            return StorageRepository(decode_files(memory_files))
        else:
            raise RuntimeError('Unable to find memory files.')

def update_source(new_files):
    try:
        shutil.rmtree('data')
        os.mkdir('data')
    except:
        os.mkdir('data')
    for file in new_files:
            print(file)
            write_csv_file(file)

def write_csv_file(file_url):
    try:
        response = requests.get(file_url, verify=False, stream=True) 
        year = file_url.split('/')[len(file_url.split('/')) - 2]  
        file_name = 'data/{}'.format(file_url.split('/')[len(file_url.split('/')) - 1].replace('.csv', '-{}.csv'.format(year)))  
        response.raw.decode_content = True

        with open(file_name, 'wb') as f:
            shutil.copyfileobj(response.raw, f)
    except Exception as ex:
        print("Error to write csv file.", ex)

@click.group()
def cli():
    get_map()
    requests.packages.urllib3.disable_warnings(InsecureRequestWarning)
    pass

@cli.command()
def build():
    stats = time()
    map = glob.glob("data/*.csv")
    if len(map) != 0:
        stats = time() - stats
        click.echo(f'Route map built in {stats:.1f}s.')
        with open('.map', 'wb') as file:
                pickle.dump(map, file)
    else:
        return click.echo(f'Unable to find memory files, please use update_storage command.')

@cli.command()
@click.argument('team')
def statistics(team):
    stats = time()
    statistics = Statistics(team)
    stats = time() - stats
    click.echo(f'Get Statistics in {stats:.1f}s.')
    click.echo('\n====================')
    click.echo(f'''{statistics.team_name} STATISTICS:
        Team Name: {statistics.team_name}.
        Total Matches: {statistics.total_matches}.
        Total Matches Winner: {statistics.total_matches_winner}.
        Total Matches Draw: {statistics.total_matches_draw}.
        Total Matches Losed: {statistics.total_matches_lose}.
        ''')

@cli.command()
def storage():
        map = get_map()
        click.echo(map.storage)

@cli.command()
@click.argument('country')
@click.option('--url', default="https://www.football-data.co.uk")
def update(url, country):    
        if country.lower() == 'portugal':
            url += "/portugalm.php"
        stats = time()
        update_source(get_files(url))
        stats = time() - stats
        click.echo(f'Updated in {stats:.1f}s, please make build command.')

if __name__ == '__main__':
    cli()



from storage import StorageRepository

class MatchLine():
    def __init__(self, line):
        self.league = line[0]
        self.date = line[1]
        self.home_team = line[2]
        self.away_team = line[3]
        self.full_time_home_goals = line[4]
        self.full_time_away_goals = line[5]
        self.full_time_winner = line[6]
        if len(line) > 7:
            self.half_time_home_goals = line[7]
            self.half_time_away_goals = line[8]
            self.half_time_winner = line[9]

class TeamDomain(StorageRepository):
    def __init__(self):
        pass
    
    def parse_match_line(self, line):
        return MatchLine(line)

    def get_all_teams(self):
        return list(super().storage.keys())

    def get_all_matches_by_team_name(self, team_name):
        return super().storage[team_name.lower()]['matches']

    def get_all_home_matches_by_team_name(self, team_name):
        home_matches = []
        matches = super().storage[team_name.lower()]['matches']
        for match in matches:
            if MatchLine(match).home_team.lower() == team_name.lower():
                home_matches.append(match)
        return home_matches

    def get_all_away_matches_by_team_name(self, team_name):
        away_matches = []
        matches = super().storage[team_name.lower()]['matches']
        for match in matches:
            if MatchLine(match).away_team.lower() == team_name.lower():
                away_matches.append(match)
        return away_matches

    def get_all_home_matches_winner_by_team_name(self, team_name):
        home_matches = []
        matches = super().storage[team_name.lower()]['matches']
        for match in matches:
            if MatchLine(match).home_team.lower() == team_name.lower() and MatchLine(match).full_time_winner == "H":
                home_matches.append(match)
        return home_matches

    def get_all_home_matches_lose_by_team_name(self, team_name):
        home_matches = []
        matches = super().storage[team_name.lower()]['matches']
        for match in matches:
            if MatchLine(match).home_team.lower() == team_name.lower() and MatchLine(match).full_time_winner == "A":
                home_matches.append(match)
        return home_matches
    
    def get_all_home_matches_draw_by_team_name(self, team_name):
        home_matches = []
        matches = super().storage[team_name.lower()]['matches']
        for match in matches:
            if MatchLine(match).home_team.lower() == team_name.lower() and MatchLine(match).full_time_winner == "D":
                home_matches.append(match)
        return home_matches

    def get_all_away_matches_winner_by_team_name(self, team_name):
        away_matches = []
        matches = super().storage[team_name.lower()]['matches']
        for match in matches:
            if MatchLine(match).away_team.lower() == team_name.lower() and MatchLine(match).full_time_winner == "A":
                away_matches.append(match)
        return away_matches

    def get_all_away_matches_lose_by_team_name(self, team_name):
        away_matches = []
        matches = super().storage[team_name.lower()]['matches']
        for match in matches:
            if MatchLine(match).away_team.lower() == team_name.lower() and MatchLine(match).full_time_winner == "H":
                away_matches.append(match)
        return away_matches

    def get_all_away_matches_draw_by_team_name(self, team_name):
        away_matches = []
        matches = super().storage[team_name.lower()]['matches']
        for match in matches:
            if MatchLine(match).away_team.lower() == team_name.lower() and MatchLine(match).full_time_winner == "D":
                away_matches.append(match)
        return away_matches

    def get_all_matches_winner_by_team_name(self, team_name):
        tmatches = []
        matches = super().storage[team_name.lower()]['matches']
        for match in matches:
            if (MatchLine(match).away_team.lower() == team_name.lower() and MatchLine(match).full_time_winner == "A") or (MatchLine(match).home_team.lower() == team_name.lower() and MatchLine(match).full_time_winner == "H"):
                tmatches.append(match)
        return tmatches

    def get_all_matches_lose_by_team_name(self, team_name):
        tmatches = []
        matches = super().storage[team_name.lower()]['matches']
        for match in matches:
            if (MatchLine(match).away_team.lower() == team_name.lower() and MatchLine(match).full_time_winner == "H") or (MatchLine(match).home_team.lower() == team_name.lower() and MatchLine(match).full_time_winner == "A"):
                tmatches.append(match)
        return tmatches

    def get_all_matches_draw_by_team_name(self, team_name):
        tmatches = []
        matches = super().storage[team_name.lower()]['matches']
        for match in matches:
            if (MatchLine(match).away_team.lower() == team_name.lower() and MatchLine(match).full_time_winner == "H") or (MatchLine(match).home_team.lower() == team_name.lower() and MatchLine(match).full_time_winner == "A"):
                tmatches.append(match)
        return tmatches

    


import setuptools

setuptools.setup(
    name="football-tools",
    version="0.0.4",
    author="Tiago Vicente",
    author_email="tmavicente@gmail.com",
    description="A small football statistics tool",
    url="https://gitlab.com/tmavicente/football-tools",
    packages=setuptools.find_packages(),
    include_package_data=True,
    install_requires=[
        'Click',
    ],
    entry_points='''
        [console_scripts]
        cli=scripts.cli:cli
    ''',
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: GNU General Public License v2 (GPLv2)",
        "Operating System :: OS Independent",
    ],
)